//////////////////////////////////////////////////////
class ProductionMatrices
{
public:
  int nE_proton;
  int nE_gamma;

  valarray<double> E_proton, E_gamma;


  // original Huang matrices

  valarray<double>          prodxsection_p_matrix; // total inelastic cross-section, mb
  valarray<double>          prodxsection_he_matrix;// total inelastic cross-section, mb

  vector<valarray<double> > gamma_decay_p_matrix;  // differential gamma-ray spectrum, GeV-1
  vector<valarray<double> > gamma_decay_he_matrix; // differential gamma-ray spectrum, GeV-1


  // rebinned matrices for differential spectra

  int nE_proton_rebin;
  int nE_gamma_rebin;
  valarray<double> E_proton_rebin, E_gamma_rebin;
  vector<valarray<double> > gamma_decay_p_matrix_rebin;
  vector<valarray<double> > gamma_decay_he_matrix_rebin;

  valarray<double>          gamma_decay_p_emissivity_rebin;
  valarray<double>          gamma_decay_he_emissivity_rebin;

  // matrices from p, He spectra to gamma-ray bands
  vector<valarray<double> > gamma_band_p_matrix;
  vector<valarray<double> > gamma_band_he_matrix;

  string energy_units; // "GeV" or "MeV"  (per nucleon)
  string energy_type;  // "total_energy" or "kinetic_energy" (per nucleon)

  
  int init(string energy_type, string energy_units);

  int read();
  int print();
  int rebin(valarray<double> E_proton_rebin, valarray<double> E_gamma_rebin);

  int emissivity(valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin );

  double emissivity_band(valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin,
                         valarray<double> E_proton_rebin,
                         double E_gamma_band_min, double E_gamma_band_max, double E_gamma_factor);


  valarray<double>  emissivity_band
          (valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin,
           valarray<double> E_proton_rebin_,
           valarray<double>E_gamma_band_min, valarray<double> E_gamma_band_max, double E_gamma_factor);

  int gen_gamma_band_matrices(valarray<double> E_proton_rebin,
                              valarray<double>E_gamma_band_min, valarray<double> E_gamma_band_max, double E_gamma_factor);

  // version using gamma_band matrices precomputed with gen_gamma_band_matrices
  valarray<double>  emissivity_band
                   (valarray<double> proton_spectrum_rebin, valarray<double> helium_spectrum_rebin);                         



  int test();

 private:
  int matrices_read;
  int matrices_rebinned;

};
